//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 14/06/02; 19:16:02

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "yahtzee.h"
#include "graphic.h"
#include "extgraph.h"



//--------------------------------------------------------------------------

char * MarksStr[] = {
"Ones",
"Twos",
"Threes",
"Fours",
"Fives",
"Sixes",
"3 of a kind",
"4 of a kind",
"Full House",
"S. Straight",
"L. Straight",
"Yahtzee",
"Chance",
"Score",
"Bonus",
"Sup. Score",
"Inf. Score",
"Total Score"};

//--------------------------------------------------------------------------

TGame *Game;

//--------------------------------------------------------------------------


void InputStr (char *buffer, short maxlen)
{
	SCR_STATE ss;
	short key, i = 0;
	buffer[0] = 0;
	SaveScrState (&ss);
	do
	{
		MoveTo (ss.CurX, ss.CurY);
		printf ("%s_  ", buffer);
		// Note that two spaces are required if F_4x6 font is used
		key = ngetchx ();
		if (key >= ' ' && key <= '~' && i < maxlen) buffer[i++] = key;
		if (key == KEY_BACKSPACE && i) i--;
		buffer[i] = 0;
	} while (key != KEY_ENTER);
}
//--------------------------------------------------------------------------


void DicesToMarks(void)
{
	int a,b,c,d;
	int f1,f2;
	
	int sum = 0;
	
	for (a=0;a<5;a++)
	sum+=Game->Dices.Dice[a].Value;
	
	for (a=0;a<MARKS_COUNT;a++)
	Game->Marks2.Mark[a].Score = 0;
	
	
	// **************************************************************************************************
	// TM_NUM_1 - TM_NUM_6
	// **************************************************************************************************
	
	int Values6[6]={0,0,0,0,0,0};
	
	for (a=0;a<5;a++)
	Values6[Game->Dices.Dice[a].Value-1]+=Game->Dices.Dice[a].Value;
	
	Game->Marks2.Mark[TM_NUM_1].Score = Values6[0];
	Game->Marks2.Mark[TM_NUM_2].Score = Values6[1];
	Game->Marks2.Mark[TM_NUM_3].Score = Values6[2];
	Game->Marks2.Mark[TM_NUM_4].Score = Values6[3];
	Game->Marks2.Mark[TM_NUM_5].Score = Values6[4];
	Game->Marks2.Mark[TM_NUM_6].Score = Values6[5];
	
	
	// **************************************************************************************************
	// TM_3KIND, TM_4KIND
	// **************************************************************************************************
	
	for (a=0;a<5;a++)
	{
		b = 1;
		for (c=0;c<5;c++)
		{
			if (a!=c && Game->Dices.Dice[a].Value==Game->Dices.Dice[c].Value)
			b++;
			
			if (b==3)
			Game->Marks2.Mark[TM_3KIND].Score = sum;
			if (b==4)
			Game->Marks2.Mark[TM_4KIND].Score = sum;
			
		}
	}
	
	// **************************************************************************************************
	// TM_FULLHOUSE
	// **************************************************************************************************
	
	if (Game->Marks2.Mark[TM_3KIND].Score!=0)
	{
		
		
		for (a=0;a<5;a++)
		for (b=0;b<5;b++)
		if (a!=b && Game->Dices.Dice[a].Value!=Game->Dices.Dice[b].Value)
		{
			f1 = 1;
			f2 = 1;
			
			for (c=0;c<5;c++)
			if (a!=c && Game->Dices.Dice[a].Value==Game->Dices.Dice[c].Value)
			f1++;
			
			for (c=0;c<5;c++)
			if (b!=c && Game->Dices.Dice[b].Value==Game->Dices.Dice[c].Value)
			f2++;
			
			if ((f1==2 && f2==3) || (f1==3 && f2==2))
			Game->Marks2.Mark[TM_FULLHOUSE].Score = 25;
		}
	}
	
	// **************************************************************************************************
	// TM_STRAIGHT_S, TM_STRAIGHT_L
	// **************************************************************************************************
	
	c = 0;
	
	for (a=1;a<=6;a++)
	{
		d = 0;
		for (b=0;b<5;b++)
		{
			if (Game->Dices.Dice[b].Value==a)
			{
				c++;
				d = 1;
				break;
			}
		}
		if (d==0)
		c = 0;
		
		if (c==4)
		Game->Marks2.Mark[TM_STRAIGHT_S].Score = 30;
		if (c==5)
		Game->Marks2.Mark[TM_STRAIGHT_L].Score = 40;
	}
	
	// **************************************************************************************************
	// TM_YAHTZEE
	// **************************************************************************************************
	
	b = 0;
	for (a=0;a<5;a++)
	if (Game->Dices.Dice[0].Value==Game->Dices.Dice[a].Value)
	b++;
	
	if (b==5)
	Game->Marks2.Mark[TM_YAHTZEE].Score = 50;
	
	// **************************************************************************************************
	// TM_CHANCE
	// **************************************************************************************************
	
	Game->Marks2.Mark[TM_CHANCE].Score = sum;
	
}
//--------------------------------------------------------------------------


void RollDices(void)
{
	int a;
	
	for (a=0;a<5;a++)
	{
		if (Game->Dices.Dice[a].Selected)
		{
			Game->Dices.Dice[a].Value = random(6)+1;
			Game->Dices.Dice[a].Selected = 0;
		}
	}
	
	Game->tries++;
	
	DicesToMarks();
	
}
//--------------------------------------------------------------------------


void RollDicesFirst(void)
{
	int a;
	
	Game->tries = 0;
	
	for (a=0;a<5;a++)
	Game->Dices.Dice[a].Selected = 1;
	
	RollDices();
	
	
}
//--------------------------------------------------------------------------

void NewGame(void)
{
	int a;
	
	for (a=0;a<MARKS_COUNT;a++)
	{
		Game->Marks.Mark[a].Score = 0;
		Game->Marks.Mark[a].Used = 0;
	}
	
	Game->Marks.Mark[TM_SCORE].Used=1;
	Game->Marks.Mark[TM_BONUS].Used=1;
	Game->Marks.Mark[TM_SCORE_SUP].Used=1;
	Game->Marks.Mark[TM_SCORE_INF].Used=1;
	Game->Marks.Mark[TM_SCORE_TOTAL].Used=1;
	
	UpDateScores();
	
	Game->MarksCount = 0;
	Game->selection = 0;
	
	RollDicesFirst();
	
}
//--------------------------------------------------------------------------

void UpDateScores(void)
{
	int a,count;
	
	count = 0;
	for (a=TM_NUM_1;a<=TM_NUM_6;a++)
	count+=(Game->Marks.Mark[a].Score);
	Game->Marks.Mark[TM_SCORE].Score = count;
	
	if (Game->Marks.Mark[TM_SCORE].Score>=63)
	Game->Marks.Mark[TM_BONUS].Score = 35;
	else
	Game->Marks.Mark[TM_BONUS].Score = 0;
	
	Game->Marks.Mark[TM_SCORE_SUP].Score = 
	Game->Marks.Mark[TM_SCORE].Score+
	Game->Marks.Mark[TM_BONUS].Score;
	
	count = 0;
	for (a=TM_3KIND;a<=TM_CHANCE;a++)
	count+=(Game->Marks.Mark[a].Score);
	Game->Marks.Mark[TM_SCORE_INF].Score = count;
	
	Game->Marks.Mark[TM_SCORE_TOTAL].Score = 
	Game->Marks.Mark[TM_SCORE_SUP].Score+
	Game->Marks.Mark[TM_SCORE_INF].Score;
	
}
//--------------------------------------------------------------------------

void RefreshMark(int mark)
{
	int x,y;
	char buffer[128];
	
	if (Game->Marks.Mark[mark].Used)
	sprintf(buffer,"(%d)",Game->Marks.Mark[mark].Score);
	else
	sprintf(buffer,"%d",Game->Marks2.Mark[mark].Score);
	
	GetMarkPosition (mark,&x,&y);
	
	DrawStr(x,y,MarksStr[mark],A_NORMAL);
	DrawStr(x+45,y,buffer,A_NORMAL);
	
	
}
//--------------------------------------------------------------------------


void RefreshScreen(void)
{
	int a;
	char buffer[128];
	
	ClrScr();
	
	for (a=0;a<MARKS_COUNT;a++)
	RefreshMark(a);
	
	sprintf(buffer,"TRIES : %d/%d - MARKS : %d/13",Game->tries,TRIES_MAX,Game->MarksCount);
	DrawStr(0,STATS_TOP,buffer,A_NORMAL);
	
	RefreshDices();
	
	
}
//--------------------------------------------------------------------------

void RefreshDices(void)
{
	int a;
	unsigned short * sprite=NULL;
	
	for (a=0;a<5;a++)
	{
		
		switch (Game->Dices.Dice[a].Value)
		{
			case 1 : sprite=DICE_Image_C1;break;
			case 2 : sprite=DICE_Image_C2;break;
			case 3 : sprite=DICE_Image_C3;break;
			case 4 : sprite=DICE_Image_C4;break;
			case 5 : sprite=DICE_Image_C5;break;
			case 6 : sprite=DICE_Image_C6;break;
		}
		
		int DiceLeft=32*a+8;
		
		Sprite16_AND(DiceLeft,DICES_TOP,16,Image_White,LCD_MEM);
		Sprite16_OR(DiceLeft,DICES_TOP,16,sprite,LCD_MEM);
		
		if (Game->Dices.Dice[a].Selected)
		Sprite16_XOR(DiceLeft,DICES_TOP,16,Image_Black,LCD_MEM);
		
		
	}
	
	
}
//--------------------------------------------------------------------------

void GetMarkPosition( int mark, int *x , int *y)
{
	
	if (mark<9)
	{
		*x = VALUES_LEFT1;
		*y = 6*mark;
	}
	else
	{
		*x = VALUES_LEFT2;
		*y = 6*(mark-9);
	}
	
}
//--------------------------------------------------------------------------

// Main Function
void _main(void)
{
	
	int a,b;
	short key;
	
	TGame GameA;
	Game=&GameA;
	
	
	FontSetSys(0);
	clrscr();
	printf("Yahtzee 1.4 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	
	FontSetSys(0);
	randomize();
	
	NewGame();
	RefreshScreen();
	
	for (;;)
	{
		
		if (Game->selection<0)
		Game->selection = 0;
		if (Game->selection>(MARKS_COUNT-1))
		Game->selection = (MARKS_COUNT-1);
		
		int x=(Game->selection>=9?10:0);
		int y=(Game->selection%9);
		
		char*start_ptr=(char*)(LCD_MEM+x+30*6*y);
		char*ptr;
		
		// SELECTION
		ptr=start_ptr;
		
		for (b=0;b<6;b++)
		{
			for (a=0;a<10;a++)
			*ptr++^=255;
			ptr+=20;
		}
		// SELECTION
		
		key = ngetchx();
		
		// SELECTION
		ptr=start_ptr;
		
		for (b=0;b<6;b++)
		{
			for (a=0;a<10;a++)
			*ptr++^=255;
			ptr+=20;
		}
		// SELECTION
		
		
		if (key==KEY_LEFT)
		{
			Game->selection-=9;
		}
		else if (key==KEY_RIGHT)
		{
			Game->selection+=9;
		}
		else if (key==KEY_UP)
		{
			Game->selection--;
		}
		else if (key==KEY_DOWN)
		{
			Game->selection++;
		}
		
		else if (key==KEY_ENTER)
		{
			
			if (!Game->Marks.Mark[Game->selection].Used)
			{
				Game->Marks.Mark[Game->selection].Score = Game->Marks2.Mark[Game->selection].Score;
				Game->Marks.Mark[Game->selection].Used = 1;
				Game->MarksCount++;
				
				UpDateScores();
				
				if (Game->MarksCount==MARKS_TODO)
				{
					RefreshScreen();
					DrawStr(0,STATS_TOP+6,"GAME OVER!",A_NORMAL);
					ngetchx();
					
					clrscr();
					printf("
					GAME OVER!
					You got %d points!
					
					Press any key...",
					Game->Marks.Mark[TM_SCORE_TOTAL].Score);
					ngetchx();
					
					NewGame();
					RefreshScreen();
				}
				else
				{
					RollDicesFirst();
					RefreshScreen();
				}
				
			}
			
			
		}
		
		else if (key==KEY_CLEAR)
		{
			
			int Dices_Selected_Count = 0;
			for (a=0;a<5;a++)
			if (Game->Dices.Dice[a].Selected)
			Dices_Selected_Count++;
			
			if (Dices_Selected_Count)
			if (Game->tries<TRIES_MAX)
			{
				RollDices();
				RefreshScreen();
			}
			
		}
		
		else if (key==KEY_APPS )
		{
			off();
		}
		
		else if (key==KEY_BACKSPACE  )
		{
			NewGame();
			RefreshScreen();
		}
		
		else if (key>=KEY_F1 && key<=KEY_F5)
		{
			int DiceID = key-KEY_F1;
			Game->Dices.Dice[DiceID].Selected^=1;
			RefreshDices();
			
		}
		
		else if (key==KEY_ESC)
		{
			
			clrscr();
			
			printf("[ESC]   : Continue");
			printf("[ENTER] : Exit");
			
			while ((key = ngetchx())!=KEY_ENTER && key!=KEY_ESC);
			
			if (key==KEY_ENTER)
			break;
			
			RefreshScreen();
			
		}
		
	}
	
}
